# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorFtp.py $
# Last committed:     $Revision: 197611 $; $Author: svcdms $; $Date: 2019-07-03 10:15:11 +0200 (Wed, 03 Jul 2019) $
#
import glob
import os
import time
import pandas as pd
import altair as alt
import sys
import getopt

from hkvportal.io.services import dataportal
from io import StringIO

from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger

def monitor_ftp(settings):

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run MonitorFtp")

    # 
    secondary_root = settings["secondary_root"]
    ftp_overview = settings["ftp_overview"]
    hkvservices = settings["hkvservices"]
    database = settings["database"]
    #monitor_root = settings["monitor_root"]
    startdate_delta = int(settings["startdate_delta"])

    # get overview at which location Sensor combination status information is expected
    df_overzicht = pd.read_csv(ftp_overview, index_col=0)
    combinations = []
    for row in df_overzicht.iterrows():
        for idx, item in enumerate(row[1]):
            if item > 0:
                parId = row[1].index[idx].split('(')[1].split(')')[0]
                pair = {"locId": row[0], "parId": parId, "factor": item}
                combinations.append(pair)

    df_monitor = pd.DataFrame.from_dict(combinations)

    period_days = 1
    enddate = pd.to_datetime(pd.datetime.now().strftime('%Y-%m-%d'))
    startdate= enddate - pd.Timedelta(startdate_delta, 'D')
    
    logger.info("Run from {} until {}".format(startdate, enddate))

    ftp_report_items = []
    monitor_items = []

    for idx, item in df_monitor.iterrows():

        factor_day = float(item['factor'])

        full_path = os.path.join(secondary_root, item['parId'],item['locId'].replace('_', '/'))

        logger.info ('Get {}'.format(full_path))

        files_in_folder = glob.glob("{}/*.*".format(full_path))
        #files = sorted(files_in_folder, key = lambda file: os.path.getctime(file), reverse=True)

        items = []
        for file in files_in_folder:     

            ts = Utils.get_timestamp_from_filename(file, '(\d*)\.\w*$')
            if ts:
                try:
                    date = pd.to_datetime(ts, format='%Y%m%d%H%M')
                    size = os.path.getsize(os.path.join(full_path, file))
                    path, filename = os.path.split(file)
                    items.append({
                        'name': filename,
                        'size': size,
                        'unit': 'bytes',
                        'date': pd.Timestamp(date.timestamp() * 10**9) 
                    }) 

                    monitor_items.append({
                        'Sensor': item['parId'], 
                        'Locatie': item['locId'], 
                        'Bestand': filename, 
                        'Omvang': size, 
                        'DatumTijd': date.strftime("%Y-%m-%d %H:%M:%S")
                    })

                except :
                    logger.error('date parse error {} : {}'.format(file, ts))

        if len(items) > 0:

            df = pd.DataFrame.from_dict(items)

            start = startdate
            delta = (enddate - startdate).days
            for doy in range(delta):

                end = start + pd.to_timedelta(1, unit='d')

                freq_no = 0
                valid_freq = 0

                try:
                    dt_slice = df[(df['date'] >= start) & (df['date'] < end)]

                    freq_no = dt_slice.shape[0] / (period_days * factor_day)
                    dt_gt_0 = dt_slice[dt_slice['size'] > 0]
                    valid_freq = dt_gt_0.shape[0] / dt_slice.shape[0]
                except:
                    pass
                
                ftp_report_items.append(
                    {'Sensor': item['parId'], 
                    'Locatie': item['locId'], 
                    'Files received': '{} / {}'.format(dt_slice.shape[0], int(period_days*factor_day)), 
                    '% Received': freq_no,
                    'Valid files': valid_freq, 
                    'Date': start.strftime("%Y-%m-%d")})

                start = end

    #df_monitor = pd.DataFrame().from_dict(monitor_items)        
    #df_monitor.to_csv(os.path.join(monitor_root, 'ftp_{}.csv'.format(pd.datetime.today().strftime('%Y-%m-%d'))))

    df_rep = pd.DataFrame().from_dict(ftp_report_items)

    # double check with the dataframe with location sensor factor combinations
    for row in df_monitor.iterrows():
        l = row[1]['locId']
        p = row[1]['parId']
        
        df_slice = df_rep[(df_rep['Locatie']==l) & (df_rep['Sensor']==p)]
        
        if df_slice.shape[0] == 0:    
            # sensor location combination not existing on FTP but should be monitored
            for d in df_rep.Date.unique():
                ftp_report_items.append({
                    'Sensor': p, 
                    'Locatie': l, 
                    'Files received': '{} / {}'.format(0, int(period_days*row[1]['factor'])), 
                    '% Received': 0,
                    'Valid files': 0, 
                    'Date': d
                })   

    df_rep = pd.DataFrame().from_dict(ftp_report_items)       

    dp = dataportal()
    dp.setDataservice(dataservice=hkvservices)
    # run once the following line of code to generate the database
    #dp.createDatabase(database = database)
    
    # save data in database
    dp.setEntryDatabase(database=database,
                        key='csv_ftp_standard_data',
                        data=df_rep.to_csv(),
                        description='CSV FTP standard data')
    
    # t.b.v excel monitoring
    single_nearest = alt.selection_single(on='mouseover', empty='none')
    
    chart = alt.Chart(df_rep).mark_rect().encode(
        y=alt.Y('Date:N', sort='descending'),
        x=alt.X('Locatie:N', title=None),
        fill=alt.Fill(
        '% Received:Q',
        scale=alt.Scale(
            domain=[0,.85,1.00],
            range=['red','#FFDC00','#2ECC40']
        ),
        legend=alt.Legend(values=[0,0.50,1], title='Health', format='.0%')        
    ),
    stroke=alt.condition(single_nearest,alt.StrokeValue('black'), alt.StrokeValue('white')),
        tooltip=['Locatie:N',
                 'Sensor:N',
                 'Files received:N',
                 'Date:N',
                 alt.Tooltip('% Received:Q', format='.0%'),
                 alt.Tooltip('Valid files:Q', format='.0%')],
        facet=alt.Facet('Sensor:N', title='Status FTP files at {}'.format(pd.datetime.today().strftime('%Y-%m-%d')))
    ).resolve_scale(
        x='independent'
    ).configure_scale(
        bandPaddingInner=0.05,
        bandPaddingOuter=0.05
    ).configure_axisX(
        labelAngle=-45
    ).configure_axisY(
        titleAngle=0,
        titleAlign="left",
        titleY=-3,
        titleX=-50,
    ).configure_header(
        titleFontSize=16,
        titleAnchor='start'
    ).add_selection(
        single_nearest
    )

    # export visualizatie als een 'in-memory text stream'
    export_html = StringIO()
    chart.save(export_html, format='html')

    logger.info("{}".format('https://dmws.hkvservices.nl/dataportal/data.asmx/read2?database=dmshoutribdijk&key=monitor_ftp_standard_data&contentType=text/html'))

    # save chart in database
    dp.setEntryDatabase(database=database,
                        key='monitor_ftp_standard_data',
                        data=export_html.getvalue(),
                        description='Monitor ftp standard data')
    
    logger.info("End of MonitorFtp")

def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorFtp.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorFtp.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        # settings_file = "../connections/monitor_status_locs_sensors.json"
        settings = Utils.read_monitor_ftp_settings(settings_file)
        monitor_ftp(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
